#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <math.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_fit.h>
#include <gsl/gsl_multifit.h>
#include <gsl/gsl_blas.h>

#include "util.h"
#include "doc.h"

strct_doc* read_docs(FILE *f, int* ndocs, int* V) {

	int D;
	strct_doc *ret;
	int tmp_wordcnt[MAXV];

	memset(tmp_wordcnt, 0, sizeof(tmp_wordcnt));

	fscanf(f, "%d", &D);
	if (ndocs)
		*ndocs = D;
	printf("Reading %d documents from the input file ...\n", D);
	assert(ret = (strct_doc*)malloc(D * sizeof(struct strct_doc)));

	*V = 0;

	for(int i = 0; i < D; i++) {
		double y;
		fscanf(f, "%lf", &y);
		ret[i].y = y;
		ret[i].word_count = 0;
		char ch = '#';
		fscanf(f, "%c", &ch);
		while (ch != '#') {
			int idx, cnt;
			fscanf(f, "%d:%d", &idx, &cnt);
			if (V && idx + 1 > *V)
				*V = idx + 1;
			ret[i].word_count += cnt;
			tmp_wordcnt[idx] += cnt;
			fscanf(f, "%c", &ch);
		}
		ret[i].item_count = 0;
		for(int j = 0; j < MAXV; j++)
			if (tmp_wordcnt[j] > 0) {
				ret[i].item_count ++;
			}
		assert(ret[i].index = (int*)malloc(ret[i].item_count * sizeof(int)));
		assert(ret[i].count = (int*)malloc(ret[i].item_count * sizeof(int)));
		int p = 0;
		for(int j = 0; j < MAXV; j++)
			if (tmp_wordcnt[j] > 0) {
				ret[i].index[p] = j;
				ret[i].count[p++] = tmp_wordcnt[j];
				tmp_wordcnt[j] = 0;
			}
	}


	return ret;

}

void write_docs(strct_doc* docs, int ndocs, FILE* f) {

	printf("Writing %d docs to the output file ...\n", ndocs);

	fprintf(f, "%d\n", ndocs);
	for(int i = 0; i < ndocs; i++) {
		fprintf(f, "%lf", docs[i].y);
		for(int j = 0; j < docs[i].item_count; j++)
			fprintf(f, " %d:%d", docs[i].index[j], docs[i].count[j]);
		fprintf(f, "#\n");
	}

	printf("Writing finished #\n");

}

void init_hz(strct_doc* doc, int K, gsl_rng* rng) {

//	printf("doc->index = %d\n", (int)(doc->index));

	doc->h = gsl_vector_alloc(K);
	double sum = 0;
	for(int k = 0; k < K; k++) {
		doc->h->data[k] = gsl_ran_flat(rng, 0, 1);
		sum += doc->h->data[k];
	}
	// smoothing
	for(int k = 0; k < K; k++)
		doc->h->data[k] = (doc->h->data[k] + 1e-3/K) / (sum + 1e-3);

	assert(doc->z = (int*)malloc(doc->word_count * sizeof(int)));
	double* p;
	assert(p = (double*)malloc(K * sizeof(double)));
	for(int k = 0; k < K; k++)
		p[k] = 1.0;
	gsl_ran_discrete_t* g = gsl_ran_discrete_preproc(K, p);
	for(int i = 0; i < doc->word_count; i++)
		doc->z[i] = (int)gsl_ran_discrete(rng, g);

	free(p);

//	printf("doc->index = %d\n", (int)(doc->index));
}

double* ols_predict_y_using_h(strct_doc* train_docs, int train_ndocs, strct_doc* test_docs, int test_ndocs, int K) {

	// preparing the training set
	gsl_matrix* xtrain = gsl_matrix_alloc(train_ndocs, K);
	gsl_vector* ytrain = gsl_vector_alloc(train_ndocs);
	for(int i = 0; i < train_ndocs; i++) {
		ytrain->data[i] = train_docs[i].y;
		for(int k = 0; k < K; k++)
			xtrain->data[ADDR2D(i, k, K)] = train_docs[i].h->data[k];
	}

	// ordinary least regression
	gsl_vector* c = gsl_vector_alloc(K);
	gsl_matrix* cov = gsl_matrix_alloc(K, K);
	double chisq = 0;
	gsl_multifit_linear_workspace* workspace = gsl_multifit_linear_alloc(train_ndocs, K);
	gsl_multifit_linear(xtrain, ytrain, c, cov, &chisq, workspace);

	// predict
	double *yret;
	assert(yret = (double*)malloc(test_ndocs * sizeof(double)));
	gsl_vector* x = gsl_vector_alloc(K);
	for(int i = 0; i < test_ndocs; i++) {
		for(int k = 0; k < K; k++)
			x->data[k] = test_docs[i].h->data[k];
		double yerr = 0;
		gsl_multifit_linear_est(x, c, cov, &(yret[i]), &yerr);
	}
	gsl_vector_free(x);

	gsl_matrix_free(xtrain);
	gsl_vector_free(ytrain);
	gsl_vector_free(c);
	gsl_matrix_free(cov);
	gsl_multifit_linear_free(workspace);

	for(int i = 0; i < test_ndocs; i++)
		test_docs[i].yhat = yret[i];
	return yret;

}

double* ols_predict_y_using_z(strct_doc* train_docs, int train_ndocs, strct_doc* test_docs, int test_ndocs, int K) {

    gsl_vector* barz = gsl_vector_alloc(K);

	// preparing the training set
	gsl_matrix* xtrain = gsl_matrix_alloc(train_ndocs, K);
	gsl_vector* ytrain = gsl_vector_alloc(train_ndocs);
	for(int i = 0; i < train_ndocs; i++) {
	    int m = train_docs[i].word_count;
		ytrain->data[i] = train_docs[i].y;
		gsl_vector_set_all(barz, 0);
		for(int j = 0; j < train_docs[i].word_count; j++)
		    barz->data[train_docs[i].z[j]] += 1.0 / m;
		for(int k = 0; k < K; k++)
			xtrain->data[ADDR2D(i, k, K)] = barz->data[k];
	}

	// ordinary least regression
	gsl_vector* c = gsl_vector_alloc(K);
	gsl_matrix* cov = gsl_matrix_alloc(K, K);
	double chisq = 0;
	gsl_multifit_linear_workspace* workspace = gsl_multifit_linear_alloc(train_ndocs, K);
	gsl_multifit_linear(xtrain, ytrain, c, cov, &chisq, workspace);

	// predict
	double *yret;
	assert(yret = (double*)malloc(test_ndocs * sizeof(double)));
	gsl_vector* x = gsl_vector_alloc(K);
	for(int i = 0; i < test_ndocs; i++) {
	    int m = test_docs[i].word_count;
	    gsl_vector_set_all(barz, 0);
	    for(int j = 0; j < test_docs[i].word_count; j++)
	        barz->data[test_docs[i].z[j]] += 1.0 / m;
		for(int k = 0; k < K; k++)
			x->data[k] = barz->data[k];
		double yerr = 0;
		gsl_multifit_linear_est(x, c, cov, &(yret[i]), &yerr);
	}
	gsl_vector_free(x);

	gsl_matrix_free(xtrain);
	gsl_vector_free(ytrain);
	gsl_vector_free(c);
	gsl_matrix_free(cov);
	gsl_multifit_linear_free(workspace);
	
	gsl_vector_free(barz);

	for(int i = 0; i < test_ndocs; i++)
		test_docs[i].yhat = yret[i];
	return yret;

}
