#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <math.h>
#include <unistd.h>
#include <sys/time.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_matrix.h>

#include "util.h"
#include "doc.h"
#include "slda.h"
#include "evaluate.h"
#include "gibbsslda.h"
#include "specslda.h"

void script1(int K, int use_initialization) {

	int ndocs, V;

	FILE *f = fopen("docs.txt", "r");
	strct_doc* docs = read_docs(f, &ndocs, &V);
	fclose(f);

	double offset = 0;
	for(int i = 0; i < ndocs; i++) offset += docs[i].y;
	offset /= ndocs;
	for(int i = 0; i < ndocs; i++) docs[i].y -= offset;

	int n1 = 10000;
	int n0 = ndocs - n1;

	double delta = DEFAULT_DELTA;
	double rho = DEFAULT_RHO;
	double *alpha, *beta;
	assert(alpha = (double*)malloc(K * sizeof(double)));
	assert(beta = (double*)malloc(V * sizeof(double)));
	for(int k = 0; k < K; k++)
		alpha[k] = DEFAULT_ALPHA;
	for(int i = 0; i < V; i++)
		beta[i] = DEFAULT_BETA;
	
	puts("Start gibbsslda fitting ...");

	strct_slda* init_slda = NULL;
	if (use_initialization) {
		FILE *f = fopen("slda.model", "r");
		init_slda = read_slda(f);
		fclose(f);
	}

	strct_slda* slda = gibbsslda_fit(docs + n1, n0, K, V, alpha, beta, delta, rho, init_slda, 1e-3);

	f = fopen("gibbs_slda.model", "w");
	write_slda(slda, f);
	fclose(f);

	puts("Start predicting ...");
	double* ys = gibbsslda_predict(docs, n1, delta, slda, 1e-3);
	f = fopen("predict.txt", "w");
	for(int i = 0; i < n1; i++)
		fprintf(f, "%.2lf\t %.2lf\n", docs[i].y, ys[i]);
	fclose(f);

	double mse = evaluate_pr(docs, n1, ys, "mse");
	printf("mse = %lf\n", mse);
	double pr2 = evaluate_pr(docs, n1, ys, "pr2");
	printf("pr2 = %lf\n", pr2);
	
	double yavg = 0;
	for(int i = 0; i < n1; i++)	
		yavg += docs[i].y;
	yavg /= n1;
	for(int i = 0; i < n1; i++)
		ys[i] = yavg;
	double mse2 = evaluate_pr(docs, n1, ys, "mse");
	printf("mse of random guess = %lf\n", mse2);

	double perword_logprob = evaluate_perword_likelihood(docs, n1, slda);
	printf("perword_logprob = %lf\n", perword_logprob);

	puts("Start OLS fitting ...");
	double* ys_ols = ols_predict_y_using_h(docs + n1, n0, docs, n1, K);
	double mse_ols = evaluate_pr(docs, n1, ys_ols, "mse");
	printf("mse_ols = %lf\n", mse_ols);
	double pr2_ols = evaluate_pr(docs, n1, ys_ols, "pr2");
	printf("pr2_ols = %lf\n", pr2_ols);

}

void script2() {

	gsl_rng_env_setup();
	gsl_rng* rng = gsl_rng_alloc(gsl_rng_default);

	int K = 50;
	int V, ndocs;
	FILE* f;

	f = fopen("ref_slda.model", "r");
	strct_slda* ref_slda = read_slda(f);
	fclose(f);

	f = fopen("gen_docs.txt", "r");
	strct_doc* docs = read_docs(f, &ndocs, &V);
	fclose(f);

	puts("Start spec-slda fitting");
	double alpha0 = gsl_blas_dasum(ref_slda->alpha);
	int L = 200;
	int T = 200;
	strct_slda* slda = specslda_fit(docs, ndocs, alpha0, K, V, L, T);
	puts("spec-slda fitting finished");

/*	puts("I am here");
	printf("slda->K = %d, slda->V = %d\n", slda->K, slda->V);
	printf("slda->alpha = %d, slda->eta = %d, slda->mu = %d\n", (int)(slda->alpha), (int)(slda->eta), (int)(slda->mu));*/

	f = fopen("slda.model", "w");
	write_slda(slda, f);
	fclose(f);

	int* perm;
	assert(perm = (int*)malloc(K * sizeof(int)));
/*	for(int k = 0; k < slda->K; k++)
		for(int i = 0; i < slda->V; i++)
			slda->mu[k]->data[i] = 1.0/slda->V;*/
	double alphaacc, etaacc, muacc;
	evaluate_slda(slda, ref_slda, 1, perm, &alphaacc, &etaacc, &muacc);
	printf("1-norm accuracy (a, e, m): %lf, %lf, %lf\n", alphaacc, etaacc, muacc);
	evaluate_slda(slda, ref_slda, 2, perm, &alphaacc, &etaacc, &muacc);
	printf("2-norm accuracy (a, e, m): %lf, %lf, %lf\n", alphaacc, etaacc, muacc);

}

void script3() {

	gsl_rng_env_setup();
	gsl_rng* rng = gsl_rng_alloc(gsl_rng_default);
	
	FILE *f;

	int K = 20;
	int V = 500;
	int M = 500;
	int ndocs = 10000;
	double sigma = 0.1;

	puts("Generating slda ...");

	strct_slda* ref_slda = alloc_slda(K, V);
	init_slda(ref_slda, rng);
	f = fopen("ref_slda.model", "w");
	write_slda(ref_slda, f);
	fclose(f);

	puts("Generating documents ...");
	strct_doc* docs = gen_docs_from_slda(ref_slda, ndocs, M, sigma, rng);
	f = fopen("gen_docs.txt", "w");
	write_docs(docs, ndocs, f);
	fclose(f);

}

void script4(int K) {

	gsl_rng_env_setup();
	gsl_rng* rng = gsl_rng_alloc(gsl_rng_default);

	int V, ndocs;
	FILE* f;

	f = fopen("docs.txt", "r");
	strct_doc* docs = read_docs(f, &ndocs, &V);
	fclose(f);

	double offset = 0;
	for(int i = 0; i < ndocs; i++) offset += docs[i].y;
	offset /= ndocs;
	for(int i = 0; i < ndocs; i++) docs[i].y -= offset;

	int n1 = 10000;
	int n0 = ndocs - n1;

	puts("Start spec-slda fitting");
	double alpha0 = DEFAULT_ALPHA * K;
	int L = 200;
	int T = 200;
	strct_slda* slda = specslda_fit(docs + n1, n0, alpha0, K, V, L, T);
	puts("spec-slda fitting finished");

	f = fopen("slda.model", "w");
	write_slda(slda, f);
	fclose(f);

/*	f = fopen("slda.model", "r");
	strct_slda *slda = read_slda(f);
	fclose(f);*/

	puts("Start predicting ...");
	double delta = DEFAULT_DELTA;
	double* ys = gibbsslda_predict(docs, n1, delta, slda, 1e-3);
	f = fopen("predict.txt", "w");
	for(int i = 0; i < n1; i++)
		fprintf(f, "%.2lf\t %.2lf\n", docs[i].y, ys[i]);
	fclose(f);

	double mse = evaluate_pr(docs, n1, ys, "mse");
	printf("mse = %lf\n", mse);
	double pr2 = evaluate_pr(docs, n1, ys, "pr2");
	printf("pr2 = %lf\n", pr2);
	
	double yavg = 0;
	for(int i = 0; i < n1; i++)	
		yavg += docs[i].y;
	yavg /= n1;
	for(int i = 0; i < n1; i++)
		ys[i] = yavg;
	double mse2 = evaluate_pr(docs, n1, ys, "mse");
	printf("mse of random guess = %lf\n", mse2);

	double perword_logprob = evaluate_perword_likelihood(docs, n1, slda);
	printf("perword_logprob = %lf\n", perword_logprob);

/*	puts("Start OLS fitting ...");
	double* ys_ols = ols_predict_y_using_h(docs + n1, n0, docs, n1, K);
	double mse_ols = evaluate_pr(docs, n1, ys_ols, "mse");
	printf("mse_ols = %lf\n", mse_ols);
	double pr2_ols = evaluate_pr(docs, n1, ys_ols, "pr2");
	printf("pr2_ols = %lf\n", pr2_ols);*/

}

double compute_mean(double x[], int n) {
	double ret = 0;
	for(int i = 0; i < n; i++)
		ret += x[i];
	return ret / n;
}

double compute_std(double x[], int n) {
	double mean = 0;
	for(int i = 0; i < n; i++)
		mean += x[i];
	mean /= n;
	double ret = 0;
	for(int i = 0; i < n; i++)
		ret += SQR(x[i] - mean);
	return sqrt(ret / n);
}

void repredict_y(strct_doc* docs, int ndocs, strct_slda* slda, double* yhat) {

	int K = slda->K;

	gsl_vector* barz = gsl_vector_alloc(K);
	for(int i = 0; i < ndocs; i++) {
		gsl_vector_set_all(barz, 0);
		for(int j = 0; j < docs[i].word_count; j++)
			barz->data[docs[i].z[j]] += 1.0 / docs[i].word_count;
		gsl_blas_ddot(slda->eta, barz, &(yhat[i]));
	}
	gsl_vector_free(barz);

}

void script5(int K, int fold, double alpha_i) {

	double logprob[3][MAXFOLD], mse[3][MAXFOLD], pr2[3][MAXFOLD], rt[3][MAXFOLD];

	int ndocs = 0;
	int V = 0;

	// Read data
	FILE* f = fopen("docs.txt", "r");
	strct_doc* docs = read_docs(f, &ndocs, &V);
	fclose(f);

	int n0 = ndocs / fold;
	int n1 = ndocs - n0;

	double *alpha, *beta;
	assert(alpha = (double*)malloc(K * sizeof(double)));
	assert(beta = (double*)malloc(V * sizeof(double)));
	for(int k = 0; k < K; k++)
		alpha[k] = alpha_i;
	for(int i = 0; i < V; i++)
		beta[i] = DEFAULT_BETA;

	double delta = DEFAULT_DELTA;
	double rho = DEFAULT_RHO;

	int L = 100;
	int T = 100;

	struct timeval start_time, end_time;

	char filename[1000];

	for(int r = 0; r < fold; r++) {

		// prepare the data
		for(int i = 0; i < n0; i++) {
			struct strct_doc t_doc = docs[i];
			docs[i] = docs[i + r * n0];
			docs[i + r * n0] = t_doc;
		}

		// Part 1
		gettimeofday(&start_time, NULL);
		strct_slda* slda1 = gibbsslda_fit(docs + n0, n1, K, V, alpha, beta, delta, rho, NULL, 1e-3);
		gettimeofday(&end_time, NULL);
		double* y1 = gibbsslda_predict(docs, n0, delta, slda1, 1e-3);

		// store the model
		memset(filename, 0, sizeof(filename));
		sprintf(filename, "slda1_K%d_alpha%.2lf_r%d.model", K, alpha_i, r);
		f = fopen(filename, "w");
		write_slda(slda1, f);
		fclose(f);
		
		rt[0][r] = end_time.tv_sec - start_time.tv_sec;
		mse[0][r] = evaluate_pr(docs, n0, y1, "mse");
		pr2[0][r] = evaluate_pr(docs, n0, y1, "pr2");
		logprob[0][r] = evaluate_perword_likelihood(docs, n0, slda1);

		for(int i = 0; i < ndocs; i++) {
			if (docs[i].h) {
				gsl_vector_free(docs[i].h);
				free(docs[i].z);
				docs[i].h = NULL;
				docs[i].z = NULL;
			}
		}
		free(y1);

		// Part 2
		gettimeofday(&start_time, NULL);
		strct_slda* slda2 = specslda_fit(docs + n0, n1, alpha_i * K, K, V, L, T);
		gettimeofday(&end_time, NULL);
		double* y2 = gibbsslda_predict(docs, n0, delta, slda2, 1e-3);

		memset(filename, 0, sizeof(filename));
		sprintf(filename, "slda2_K%d_alpha%.2lf_r%d.model", K, alpha_i, r);
		f = fopen(filename, "w");
		write_slda(slda2, f);
		fclose(f);

		rt[1][r] = end_time.tv_sec - start_time.tv_sec;
		mse[1][r] = evaluate_pr(docs, n0, y2, "mse");
		pr2[1][r] = evaluate_pr(docs, n0, y2, "pr2");
		logprob[1][r] = evaluate_perword_likelihood(docs, n0, slda2);
		
		for(int i = 0; i < ndocs; i++) {
			if (docs[i].h) {
				gsl_vector_free(docs[i].h);
				free(docs[i].z);
				docs[i].h = NULL;
				docs[i].z = NULL;
			}
		}
		free(y2);

		// Part 3

		// reset alpha
		for(int k = 0; k < K; k++)
			slda2->alpha->data[k] = DEFAULT_ALPHA;

		strct_slda* slda3 = gibbsslda_fit(docs + n0, n1, K, V, alpha, beta, delta, rho, slda2, 1e-3);
		double* y3 = gibbsslda_predict(docs, n0, delta, slda3, 1e-3);

		memset(filename, 0, sizeof(filename));
		sprintf(filename, "slda3_K%d_alpha_%.2lf_r%d.model", K, alpha_i, r);
		f = fopen(filename, "w");
		write_slda(slda3, f);
		fclose(f);

		rt[2][r] = 0;
		mse[2][r] = evaluate_pr(docs, n0, y3, "mse");
		pr2[2][r] = evaluate_pr(docs, n0, y3, "pr2");
		logprob[2][r] = evaluate_perword_likelihood(docs, n0, slda3);

		for(int i = 0; i < ndocs; i++) {
			if (docs[i].h) {
				gsl_vector_free(docs[i].h);
				free(docs[i].z);
				docs[i].h = NULL;
				docs[i].z = NULL;
			}
		}
		
		free(y3);

	}

	puts("==================================================================");
	puts("rt, mse, pr2, logprob");
	for(int i = 0; i < 3; i++) {
		switch (i) {
			case 0: printf("gibbsslda: "); break;
			case 1: printf("specslda: "); break;
			case 2: printf("spec + gibbsslda: "); break;
		}
		printf("%.3lf(%.3lf)\t", compute_mean(rt[i], fold), compute_std(rt[i], fold));
		printf("%.3lf(%.3lf)\t", compute_mean(mse[i], fold), compute_std(mse[i], fold));
		printf("%.3lf(%.3lf)\t", compute_mean(pr2[i], fold), compute_std(pr2[i], fold));
		printf("%.3lf(%.3lf)\n", compute_mean(logprob[i], fold), compute_std(logprob[i], fold));
	}
	puts("===================================================================");

}

void script6(int K, int fold, double alpha_i) {

	double rt;
	double mse[MAXFOLD], pr2[MAXFOLD], logprob[MAXFOLD];
	
	int ndocs = 0;
	int V = 0;

	// Read data
	FILE* f = fopen("docs.txt", "r");
	strct_doc* docs = read_docs(f, &ndocs, &V);
	fclose(f);

	int n0 = 100000 / fold;
	int n1 = 100000 - n0;
	int n2 = 500000 - (n0 + n1);
	int n3 = 10000;

	double *alpha, *beta;
	assert(alpha = (double*)malloc(K * sizeof(double)));
	assert(beta = (double*)malloc(V * sizeof(double)));
	for(int k = 0; k < K; k++)
		alpha[k] = alpha_i;
	for(int i = 0; i < V; i++)
		beta[i] = DEFAULT_BETA;

	double delta = DEFAULT_DELTA;
	double rho = DEFAULT_RHO;

	int L = 100;
	int T = 100;

	struct timeval start_time, end_time;

	char filename[1000];
	
	// first, train a spectral slda model
	gettimeofday(&start_time, NULL);
	strct_slda* slda_spec = specslda_fit(docs + n0 + n1, n2, alpha_i * K, K, V, L, T);
	// reset alpha
    for(int k = 0; k < K; k++)
		slda_spec->alpha->data[k] = alpha_i;
	strct_slda* slda_gibbs = gibbsslda_fit(docs + n0 + n1, n3, K, V, alpha, beta, delta, rho, slda_spec, 1e-3); 
	gettimeofday(&end_time, NULL);
	
	rt = end_time.tv_sec - start_time.tv_sec;
	
	memset(filename, 0, sizeof(filename));
	sprintf(filename, "slda_spec_K%d_alpha%.2lf.model", K, alpha_i);
	f = fopen(filename, "w");
	write_slda(slda_spec, f);
	fclose(f);

	memset(filename, 0, sizeof(filename));
	sprintf(filename, "slda_gibbs_K%d_alpha%.2lf.model", K, alpha_i);
	f = fopen(filename, "w");
	write_slda(slda_gibbs, f);
	fclose(f);
	

	for(int r = 0; r < fold; r++) {

		// prepare the data
		for(int i = 0; i < n0; i++) {
			struct strct_doc t_doc = docs[i];
			docs[i] = docs[i + r * n0];
			docs[i + r * n0] = t_doc;
		}

		double *y = gibbsslda_predict(docs, n0, delta, slda_gibbs, 1e-3);

		mse[r] = evaluate_pr(docs, n0, y, "mse");
		pr2[r] = evaluate_pr(docs, n0, y, "pr2");
		logprob[r] = evaluate_perword_likelihood(docs, n0, slda_gibbs);

		free(y);

	}

	puts("==================================================================");
	printf("rt = %.3lf\n", rt);
	printf("mse: %.3lf (%.3lf)\n", compute_mean(mse, fold), compute_std(mse, fold));
	printf("pr2: %.3lf (%.3lf)\n", compute_mean(pr2, fold), compute_std(pr2, fold));
	printf("logprob: %.3lf (%.3lf)\n", compute_mean(logprob, fold), compute_std(logprob, fold));
	puts("===================================================================");

}

void script7(int K, int fold, double alpha_i) {

    double mse[MAXFOLD], pr2[MAXFOLD];
	
	int ndocs = 0;
	int V = 0;

	// Read data
	FILE* f = fopen("docs.txt", "r");
	strct_doc* docs = read_docs(f, &ndocs, &V);
	fclose(f);

    int n0 = ndocs / fold;
    int n1 = ndocs - n0;

	double *alpha, *beta;
	assert(alpha = (double*)malloc(K * sizeof(double)));
	assert(beta = (double*)malloc(V * sizeof(double)));
	for(int k = 0; k < K; k++)
		alpha[k] = alpha_i;
	for(int i = 0; i < V; i++)
		beta[i] = DEFAULT_BETA;

	double delta = DEFAULT_DELTA;
	double rho = DEFAULT_RHO;

	int L = 100;
	int T = 100;

	char filename[1000];
	
	// first, train a spectral slda model
	strct_slda* slda_spec = specslda_fit(docs, ndocs, alpha_i * K, K, V, L, T);
	
	memset(filename, 0, sizeof(filename));
	sprintf(filename, "slda_spec_K%d_alpha%.2lf.model", K, alpha_i);
	f = fopen(filename, "w");
	write_slda(slda_spec, f);
	fclose(f);
	
	// reset alpha
    for(int k = 0; k < K; k++)
		slda_spec->alpha->data[k] = alpha_i;

    double* y0 = gibbsslda_predict(docs, ndocs, delta, slda_spec, 1e-3);
	free(y0);

	for(int r = 0; r < fold; r++) {

		// prepare the data
		for(int i = 0; i < n0; i++) {
			struct strct_doc t_doc = docs[i];
			docs[i] = docs[i + r * n0];
			docs[i + r * n0] = t_doc;
		}

		double* y = ols_predict_y_using_z(docs + n0, n1, docs, n0, K);

		mse[r] = evaluate_pr(docs, n0, y, "mse");
		pr2[r] = evaluate_pr(docs, n0, y, "pr2");

		free(y);

	}

	puts("==================================================================");
	printf("mse: %.3lf (%.3lf)\n", compute_mean(mse, fold), compute_std(mse, fold));
	printf("pr2: %.3lf (%.3lf)\n", compute_mean(pr2, fold), compute_std(pr2, fold));
	puts("===================================================================");	

}

void script8(int K, int n) {

	int ndocs;
	int V;

	FILE* f = fopen("docs.txt", "r");
	strct_doc* docs = read_docs(f, &ndocs, &V);
	fclose(f);
	assert(n <= ndocs);

	struct timeval start_time, end_time;

	double alpha_i = 0.1;

	double *alpha, *beta;
	assert(alpha = (double*)malloc(K * sizeof(double)));
	assert(beta = (double*)malloc(V * sizeof(double)));
	for(int k = 0; k < K; k++)
		alpha[k] = alpha_i;
	for(int i = 0; i < V; i++)
		beta[i] = DEFAULT_BETA;

	int L = 100;
	int T = 100;

	double delta = DEFAULT_DELTA;
	double rho = DEFAULT_RHO;

	gettimeofday(&start_time, NULL);
	specslda_fit(docs, n, alpha_i * K, K, V, L, T); 
	gettimeofday(&end_time, NULL);
	double rt_spec = end_time.tv_sec - start_time.tv_sec;

	gettimeofday(&start_time, NULL);
	gibbsslda_fit(docs, n, K, V, alpha, beta, delta, rho, NULL, 1e-3);
	gettimeofday(&end_time, NULL);
	double rt_gibbs = end_time.tv_sec - start_time.tv_sec;

	puts("===========================================");
	printf("spec rt = %.3lf\n", rt_spec);
	printf("gibbs rt = %.3lf\n", rt_gibbs);

}

void script9(int K, int fold, int n, double alpha_i) {

    double mse[2][MAXFOLD], pr2[2][MAXFOLD], logprob[2][MAXFOLD];
	
	int ndocs = 0;
	int V = 0;

	// Read data
	FILE* f = fopen("docs.txt", "r");
	strct_doc* docs = read_docs(f, &ndocs, &V);
	fclose(f);

    int n0 = 100000 / fold;
    int n1 = 100000 - n0;
    int n2 = ndocs - 100000;

	double *alpha, *beta;
	assert(alpha = (double*)malloc(K * sizeof(double)));
	assert(beta = (double*)malloc(V * sizeof(double)));
	for(int k = 0; k < K; k++)
		alpha[k] = alpha_i;
	for(int i = 0; i < V; i++)
		beta[i] = DEFAULT_BETA;

	double delta = DEFAULT_DELTA;
	double rho = DEFAULT_RHO;

	int L = 100;
	int T = 100;

	char filename[1000];
	
	// first, train a spectral slda model
	strct_slda* slda_spec = specslda_fit(docs + n0 + n1, n2, alpha_i * K, K, V, L, T);
	
	memset(filename, 0, sizeof(filename));
	sprintf(filename, "slda_spec_K%d_alpha%.2lf.model", K, alpha_i);
	f = fopen(filename, "w");
	write_slda(slda_spec, f);
	fclose(f);
	
	// reset alpha
    for(int k = 0; k < K; k++)
		slda_spec->alpha->data[k] = alpha_i;
		
	strct_slda* slda1 = gibbsslda_fit(docs + n0 + n1, n, K, V, alpha, beta, delta, rho, NULL, 1e-3);
	strct_slda* slda2 = gibbsslda_fit(docs + n0 + n1, n, K, V, alpha, beta, delta, rho, slda_spec, 1e-3);

	for(int r = 0; r < fold; r++) {
		
		double* y1 = gibbsslda_predict(docs + r * n0, n0, delta, slda1, 1e-3);
		mse[0][r] = evaluate_pr(docs + r * n0, n0, y1, "mse");
		pr2[0][r] = evaluate_pr(docs + r * n0, n0, y1, "pr2");
		logprob[0][r] = evaluate_perword_likelihood(docs + r * n0, n0, slda1);
		free(y1);
		
		double* y2 = gibbsslda_predict(docs + r * n0, n0, delta, slda2, 1e-3);
		mse[1][r] = evaluate_pr(docs + r * n0, n0, y2, "mse");
		pr2[1][r] = evaluate_pr(docs + r * n0, n0, y2, "pr2");
		logprob[1][r] = evaluate_perword_likelihood(docs + r * n0, n0, slda2);
		free(y2);

	}

	puts("==================================================================");
	printf("gibbs mse: %.3lf (%.3lf)\n", compute_mean(mse[0], fold), compute_std(mse[0], fold));
	printf("gibbs pr2: %.3lf (%.3lf)\n", compute_mean(pr2[0], fold), compute_std(pr2[0], fold));
	printf("gibbs logprob: %.3lf (%.3lf)\n", compute_mean(logprob[0], fold), compute_std(logprob[0], fold));
	printf("combined mse: %.3lf (%.3lf)\n", compute_mean(mse[1], fold), compute_std(mse[1], fold));
	printf("combined pr2: %.3lf (%.3lf)\n", compute_mean(pr2[1], fold), compute_std(pr2[1], fold));
	printf("combined logprob: %.3lf (%.3lf)\n", compute_mean(logprob[1], fold), compute_std(logprob[1], fold));	
	puts("===================================================================");	

}

int main(int argc, char* argv[]) {

	int K = 50;
	int fold = 0;
	double alpha = 0.05;
	if (argc >= 2) {
		sscanf(argv[1], "%d", &K);
	}
	printf("K = %d\n", K);
	if (argc >= 3) {
		sscanf(argv[2], "%d", &fold);
		printf("fold = %d\n", fold);
	}
	if (argc >= 4) {
		sscanf(argv[3], "%lf", &alpha);
		printf("alpha = %lf\n", alpha);
	}

	int n = 0;
	sscanf(argv[4], "%d", &n);
	printf("n = %d\n", n);

	script9(K, fold, n, alpha);

	return 0;

}
